function [ y_out] = MyInvertSonogram( msgram,fftsize,m_L,m_S )

numFrames=size(msgram,2);
numBins = size(msgram,1);

% if sonogram include a frequency=0 bin, lose it    
if (numBins  == fftsize/2 + 1)  
    fprintf('size of sgram is %d \n', numBins);
    fprintf('Throwing out bin 0!');
    msgram([fftsize/2 + 1],:)=[];
    numBins = size(msgram,1);
    fprintf('now number of bins is %d\n', numBins)
end

m_phase=zeros(fftsize/2,1); %phase
m_collect=zeros(fftsize,1); %temporary_input collection of output array_input
[m_win]=windowFunction(fftsize);  %windowing function

mwin=m_win';    %transpose window 

%this processes one chunk of audio at a time, as per fftsize
      
 for i=0:numFrames-1
            
            m_mag=msgram(:,i+1);
            
        %   check through the magnitude spectrum to find peak
            for j=2:1:fftsize/2-1
                if(m_mag(j)>m_mag(j-1) && m_mag(j)>m_mag(j+1))

%  Phase estimate using quadratic interpolation as per Julios O. Smith
%  http://www.dsprelated.com/dspbooks/sasp/Quadratic_Interpolation_Spectral_Peaks.html
%  Use quadratic interpolation to estimate the real peak position

            alpha=m_mag(j-1);
            beta=m_mag(j);
            gamma=m_mag(j+1);
            denom=alpha-2*beta+gamma;


            % initialize p
            
            if(denom~=0)
            p=0.5*(alpha-gamma)/denom;
            else
            p=0;
            end

        % checking for peaks

        phaseRate=2*pi*(j-1+p)/m_L;    %adjusted phase rate
        m_phase(j)= m_phase(j) + m_S*phaseRate; %phase accumulator for this peak bin
        peakPhase=m_phase(j);
    

% Apply_input simple phase locking around the peaks.
% The phase relationships of the bins around the peak were determined by
% some simple experiments inspired by: 
% Laroche/Dolson "About This Phasiness Business" (1997), which mentions a paper
% M.S. Puckette "Phase-locked vocoder" (1995).  
% http://msp.ucsd.edu/Publications/mohonk95.pdf

% According to Laroche/Dolson:
% "Puckette in [5] recognized that for a constant-frequency_input constant-amplitude sinusoid 
% the sy_inputnthesis phases around the maximum of the Fourier transform should exhibit +/- pi 
% alternations and proposed a very_input simple way_input to constrain them to do so".


% Do 0/pi phase shift thing around the peak

% If actual peak is to the right
        if (p>0)

        % First bin to right has pi shift
            bin=j+1;
            m_phase(bin)=peakPhase+pi;
            bin=j-1;
      
            while(bin>1 && m_mag(bin)<m_mag(bin+1))

    % Bins to left have shift of pi
                
                m_phase(bin)=peakPhase+pi;
                bin=bin-1;
            end

            bin=j+2;
             while(bin<(fftsize/2) && m_mag(bin)<m_mag(bin-1))

    % Other bins (apart from the immediate bin to the right of the peak)
    % have zero shift
    
                m_phase(bin)=peakPhase+0;
                bin=bin+1;

             end        
        end
     
     
% Peak is to the left
      
     if(p<0)

    % First bin to left has pi shift
            bin=j-1;
            m_phase(bin)=peakPhase+pi;

            bin=j+1;

 % similarly, like before, instead of using bin< fftsize/2-1 
 % I use bin<fftsize/2
        while(bin<(fftsize/2) && m_mag(bin)<m_mag(bin-1))
            
% Bins to right have shift of pi
                m_phase(bin)=peakPhase+pi;
                bin=bin+1;
            
        end
         bin=j-2;
         while(bin>1 && m_mag(bin)<m_mag(bin+1))
             
% Other bins to left have zero shift
            m_phase(bin)=peakPhase+0;
            bin=bin-1;
        end
     end
                end
            end  

       m_recon=m_mag.*exp(1i*m_phase);       %reconstruct with new phase
       m_recon=[m_recon(1:fftsize/2); zeros(fftsize/2,1)];

   % Using conj for conjugate symmetry- reconstruct other half of FFT
   
    for k=2:1:fftsize/2
    m_recon(fftsize+2-k)=conj(m_recon(k));
    m_recon(fftsize+2-k)=conj(m_recon(k));
    end

    % IFFT of the result and subsequent windowing
    
        m_recon(1)=0;
        m_recon(fftsize/2+1)=0;
        m_recon=real(ifft(m_recon));
        m_recon=m_recon.*mwin;

    if(i==0)
             m_collect=m_recon; %output buffer which collects the output 
            
    end
    if(i>0)
        
        %overlap add      
            m_collect=[m_collect; zeros(m_S,1)];
            m_collect=[m_collect(1:size(m_collect,1)-fftsize); m_collect(size(m_collect,1)-fftsize+1:size(m_collect,1))+m_recon];
            
        %updating output buffer
    
    end   
 end
y_out=m_collect; 

end

