% SPSI: Single Pass Spectrogam Inversion
% see: http://anclab.org/software/phaserecon/
% cite: Beauregard, G., Harish, M. and Wyse, L. (2015), Single Pass Spectrogram Inversion, in Proceedings of the IEEE International Conference on Digital Signal Processing. Singapore, 2015.
%
% Top level script for sound - >magnitude spectrum -> reconstruction


% Use loger window for higher sample rate or sounds with low freq content
[y,Fs]=audioread('sounds/BeingRural_short.wav'); %target wavefile
fftsize=1024;                     %specify target fftsize

% For better transients, but poor low frequency reconstruction, use smaller fft size
%[y,Fs]=audioread('sounds/xylophoneCGC.wav'); %target wavefile
%fftsize=256;

y=y(:,1);
siglen = min(4*Fs, size(y));
y=y(1:siglen);

m_L=fftsize;                      %winlength
m_S=fftsize/4;                    %hop size- can be changed, fftsize/4 works well in general 

%zero pad input waveform
num=ceil(length(y)/fftsize);      %zero pad input wavefile so we use integer number of windows
y=[y; zeros(num*fftsize-length(y),1)]; 

%-------------------------------------------------
% 1. Linear Freq sonogram, and then invert
%-------------------------------------------------
sgram = Sonogram(y, fftsize, m_S);
magsgram=abs(sgram);
%do Single Pass Spectrogram Inversion
[audiorecon]=InvertSonogram(magsgram,fftsize,m_L,m_S);

fprintf('size of sgram is ')
size(sgram)

%Test the SER Ratio
fprintf('SER Ration, for what it is worth: ')
SERRatio(y,audiorecon)

figure
imagesc(log(magsgram)); axis xy; colorbar
title('Original Linear Spectrogram'); 
colormap hot
% Produce some visual and audio output

soundsc(audiorecon, 22050)
